#include <cstdio>
#include <ctime>

#include <algorithm>
#include <chrono>
#include <iostream>
#include <string>

#include "hardware/pio.h"
#include "hardware/rtc.h"

#include "pico/cyw43_arch.h"
#include "pico/stdlib.h"
#include "pico/util/datetime.h"

#include "http_client.h"
#include "ntp_client.h"
#include "utils.h"
#include "ws2812.h"

using namespace std;

#define WS2812_PIN 22

int main() {
    stdio_init_all();
    rtc_init();

#ifdef DEBUG
    printf("!!! DEBUG mode!!!\n");
#endif

    WS2812 led(WS2812_PIN);

    wifi_setup();

    NtpClient::setDateTime();
    HttpClient client;
    std::string csv("");
    for (int i = 1; i <= 10; i++) {
#ifdef DEBUG
        printf("Attempt %d for retrieving data.\n", i);
#endif
        csv = client.retrieveWasteDatesAsCsv();
        if (csv.length() > 0) {
#ifdef DEBUG
            printf("Data received!\n");
#endif
            break;
        }
    }

    if (csv.length() == 0) {
#ifdef DEBUG
        printf("Error getting data. Exiting!");
#endif
        cyw43_arch_deinit();
        return 1;
    }

    datetime_t dt;

    auto dates = parseCsv(csv);
#ifdef DEBUG
    printf("Number of Dates: %d\n", dates.size());
#endif
    while (true) {
        auto timestamp = time_us_64() + 60000000ull;

        rtc_get_datetime(&dt);

        chrono::year_month_day tomorrowYMD(chrono::year{dt.year},
                                           chrono::month{static_cast<unsigned>(dt.month)},
                                           chrono::day{static_cast<unsigned>(dt.day + 1)});
        auto it = std::find_if(dates.begin(), dates.end(), [&tomorrowYMD](const WasteDate &date) {
            return date.date == tomorrowYMD;
        });

        int8_t hour = dt.hour + 1; // MEZ
        if (isDST(dt.day, dt.month, dt.dotw))
            ++hour; // MESZ

        // If there was a waste bin pickup found AND we are in the evening (>= 18:00) …
        if (it != dates.end() && hour >= 14) {
            auto wasteDate = *it;
            size_t count{0};
            auto currentTime = time_us_64();
            while (currentTime < timestamp) {
                size_t index = count % wasteDate.wasteTypes.size();

                switch (wasteDate.wasteTypes.at(index)) {
                case Waste::GelberSack:
                    led.switchColor(Color::YELLOW);
                    break;
                case Waste::Restmuell:
                    led.switchColor(Color::RED);
                    break;
                case Waste::Papiertonne:
                    led.switchColor(Color::BLUE);
                    break;
                case Waste::Problemstoffmobil:
                    led.switchColor(Color::CYAN);
                    break;
                case Waste::Biotonne:
                    led.switchColor(Color::GREEN);
                    break;
                }

                auto timeLeft = timestamp - currentTime;

                if (timeLeft > 3000000) {
                    sleep_us(3000000);
                } else {
                    sleep_us(timeLeft);
                }

                currentTime = time_us_64();
                count++;
            }

        } else {
            led.blinkReady();
            sleep_until(static_cast<absolute_time_t>(timestamp));
        }
    }

    cyw43_arch_deinit();
}
